/*
 * objective_function_template.c
 *
 * Copyright 2013, 2014 Philippe Preux
 * Philippe.Preux@univ-lille3.fr
 *
 * This is GPL software.
 *
 * Do not modify anything except where it is indicated you type the code
 * to evaluate the objective function.
 *
 * This file is part of yaStoSOO.

    yaStoSOO is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    yaStoSOO is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with yaStoSOO.  If not, see <http://www.gnu.org/licenses/>.

 */

#ifndef _GNU_SOURCE
#define _GNU_SOURCE
#endif

#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include <assert.h>

#include "Malloc.h"

#include "stosoo_optim.h"

extern double box_muller (const double, const double);

size_t nb_calls_to_obj_fct = 0;
static size_t N;
static size_t d;
static double noise_std_dev = .1;

size_t fct_id (void) { return (-1); }
char *fct_name (void) { return "fct_name"; }

static double last_fct_eval = .0;
double last_fct_evaluation (void)
{
  return last_fct_eval;
}

void set_noise_std_dev (const double value)
{
  noise_std_dev = value;
}

static double *x_opt = NULL;

void f_init (const size_t nb_eval, const size_t dimension)
{
  N = nb_eval;
  d = dimension;

  /* Here, you type whatever you want. This is the correct way to 
     initialize your objective function if needed.
     This function call is the last stage of StoSOO initialization, 
     just before the optimization begins. */
}

double f_maximum (void)
{
  return nan ("");
}

double f (DataPoint x)
{
  if (nb_calls_to_obj_fct >= N) return nan ("");
  nb_calls_to_obj_fct ++;

  /* ***************************************** */
  /* Here, you type your objective function. 
     Do not modify anything before this point. */
  /* ***************************************** */


  /* Express yourself here! */

  /* Noise: unless you play in the real with a noisy function, 
     you probably want to add some noise at this point. To do this:

     you may compute the value of the objective function and 
     assign it to a double variable named v
     and then add the noise with a line such as:

     v *= (1. + box_muller (0., noise_std_dev));

     This line adds a Gaussian noise, 0-centered, 
     with the specified standard deviation.
  */

  /* *************************** */
  /* Do not modify what follows. */
  /* *************************** */
  return last_fct_eval = -v;
}

