/* boxmuller.c           Implements the Polar form of the Box-Muller
                         Transformation

                      (c) Copyright 1994, Everett F. Carter Jr.
                          Permission is granted by the author to use
			  this software for any application provided this
			  copyright notice is preserved.

 * Source: http://www.design.caltech.edu/erik/Misc/Gaussian.html
*/

#include <stdlib.h>
#include <math.h>

/* normal random variate generator, mean m, standard deviation s */
double box_muller (const double m, const double s)
{				        
  double x1, x2, w, y1;
  static double y2;
  static int use_last = 0;
  static double rnd_scale = 1. / ((double) RAND_MAX);
  
  if (use_last) { /* use value from previous call */
    y1 = y2;
    use_last = 0;
  } else {
    do {
      x1 = 2.0 * rand () * rnd_scale - 1.0;
      x2 = 2.0 * rand () * rnd_scale - 1.0;
      w = x1 * x1 + x2 * x2;
    } while (w >= 1.0);
    w = sqrt ((-2.0 * log (w)) / w);
    y1 = x1 * w;
    y2 = x2 * w;
    use_last = 1;
  }
  
  return (m + y1 * s);
}
