/*
 * stosoo_optim.h
 *
 * Copyright 2013, 2014 Philippe Preux
 * Philippe.Preux@univ-lille3.fr
 *
 * This is GPL software.
 *
 * This file is part of yaStoSOO.

    yaStoSOO is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    yaStoSOO is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with yaStoSOO.  If not, see <http://www.gnu.org/licenses/>.

 */

#ifndef stosoo_optim_h
#define stosoo_optim_h

typedef struct {
  size_t depth;
  double *x, *dim, *center;
  size_t nb_eval;
  double sum_eval;
  double b; /* the bound */
  size_t parent;
  size_t expanded;
  size_t split_direction;
  size_t *dim_index;
} Noeud, *P_noeud;

typedef double *DataPoint;

typedef struct {
  double x_star_val, x_star_val_preproc;
  double *x_star, *x_star_preproc;
  size_t stosoo_nb_eval;
  int post_proc_flag;
  size_t post_proc_nb_eval;
} t_les_x_star_val;

extern void yaStoSOO_set_hmax (const size_t);
extern void yaStoSOO_set_percentage (const double);
extern void yaStoSOO_set_post_process (const int);
extern void yaStoSOO_set_N_noeuds (const size_t);
extern size_t yaStoSOO_nb_nodes (void);
extern void yaStoSOO_set_split_policy (const int);
extern void yaStoSOO_set_split_factor (const size_t);
extern void yaStoSOO_set_max_number_of_evaluations_per_node (const size_t);
extern void yaStoSOO_set_number_of_obj_fct_evaluations (const size_t);
extern void yaStoSOO_set_max_depth (const size_t);
extern void yaStoSOO_set_delta (const double);
extern void yaStoSOO_set_domain (const size_t, const double *, const double *);
extern void yaStoSOO_set_soo_mode (void);
extern void yaStoSOO_optim_init (void);
extern size_t yaStoSOO_get_N_values (void);
extern void yaStoSOO_optim (void);

extern t_les_x_star_val *yaStoSOO_les_x_star_val (void);
extern double *yaStoSOO_x_star (void);
extern double yaStoSOO_x_star_val (void);

#define SPLIT_ORIGINAL 0
#define SPLIT_AT_RANDOM 1
#define SPLIT_BY_INCREASING 2
#define SPLIT_BY_DECREASING 3

#endif
